@@ -1,14 +1,13 @@
 import collections.abc
 
-from collections import Counter
 from zipfile import ZipFile, BadZipFile
 from pathlib import Path
 from typing import Sequence, overload, Union, Iterable, Iterator, Any, Optional, Tuple, Callable, Mapping, Type
 from coba.backports import Literal
 
 from coba                 import pipes
-from coba.contexts        import CobaContext, DiskCacher
-from coba.primitives      import Context, Action, HashableSparse
+from coba.contexts        import CobaContext, DiskCacher, DecoratedLogger, ExceptLog, NameLog, StampLog
+from coba.primitives      import Context, Action
 from coba.random          import CobaRandom
 from coba.pipes           import Pipes, Source, HttpSource, IterableSource, JsonDecode
 from coba.exceptions      import CobaException
@@ -24,7 +23,7 @@ from coba.environments.synthetics import KernelSyntheticSimulation, MLPSynthetic
 from coba.environments.supervised import SupervisedSimulation
 
 from coba.environments.filters   import EnvironmentFilter, Repr, Batch, Chunk, Logged, Finalize, BatchSafe
-from coba.environments.filters   import Binary, Shuffle, Take, Sparse, Reservoir, Cycle, Scale, Unbatch
+from coba.environments.filters   import Binary, Shuffle, Take, Sparse, Reservoir, Cycle, Scale, Unbatch, Slice
 from coba.environments.filters   import Impute, Where, Noise, Riffle, Sort, Flatten, Cache, Params, Grounded
 
 from coba.environments.serialized import EnvironmentFromObjects, EnvironmentsToObjects, ZipMemberToObjects, ObjectsToZipMember
@@ -291,7 +290,7 @@ class Environments(collections.abc.Sequence, Sequence[Environment]):
         #Experience has shown that most of the time we want to sort.
         #This doesn't change the experiment results. It simply makes it
         #easier monitor an experiment while it runs in the background.
-        ordered  = sorted(shuffled, key=lambda env: env.params['shuffle'])
+        ordered = sorted(shuffled, key=lambda env: env.params.get('shuffle',0))
 
         return Environments(ordered)
 
@@ -314,6 +313,10 @@ class Environments(collections.abc.Sequence, Sequence[Environment]):
     def take(self, n_interactions: int) -> 'Environments':
         """Take a fixed number of interactions from the Environments."""
         return self.filter(Take(n_interactions))
+    
+    def slice(self, start: Optional[int], stop: Optional[int]=None, step:int = 1) -> 'Environments':
+        """Take a slice of interactions from an Environment."""
+        return self.filter(Slice(start,stop,step))
 
     def reservoir(self, n_interactions: int, seeds: Union[int,Sequence[int]]) -> 'Environments':
         """Take a random fixed number of interactions from the Environments."""
@@ -394,37 +397,42 @@ class Environments(collections.abc.Sequence, Sequence[Environment]):
 
     def save(self, path: str, processes:int=1, overwrite:bool=False) -> 'Environments':
 
+        self_envs = list(self)
+
         if Path(path).exists():
             try:
                 path_envs   = Environments.from_save(path)
                 path_params = [e.params for e in path_envs]
-                self_params = [e.params for e in self     ]
-                
+                self_params = [e.params for e in self_envs]
+
                 try:
-                    for param in path_params:
-                        self_params.pop(self_params.index(param))
+                    while path_params:
+                        param_index_in_self = self_params.index(path_params.pop())
+                        self_params.pop(param_index_in_self)
+                        self_envs.pop(param_index_in_self)
                 except ValueError:
+                    #there is a param in the file that isn't in self
                     is_equal = False
                 else:
                     is_equal = True
 
-                if is_equal:
+                if is_equal and not self_envs:
                     return path_envs
-                elif overwrite:
+                if not is_equal and overwrite:
                     Path(path).unlink()
-                else:
+                if not is_equal and not overwrite:
                     raise CobaException("The Environments save file does not match the actual Environments and overwite is False.")
+            
             except BadZipFile:
                 if overwrite:
                     Path(path).unlink()
                 else:
-                    raise CobaException("The given save file appears to be corruptted. Please check it and delete if it is unusable.")
-
-        if processes == 1:
-            Pipes.join(EnvironmentsToObjects(),ObjectsToZipMember(path)).write(self)
-        else:
-            Pipes.join(CobaMultiprocessor(EnvironmentsToObjects(),processes),ObjectsToZipMember(path)).write(self)
+                    raise CobaException("The given save file appears to be corrupted. Please check it and delete if it is unusable.")
 
+        CobaContext.logger = DecoratedLogger([ExceptLog()], CobaContext.logger, [StampLog()] if processes ==1 else [NameLog(), StampLog()])
+        Pipes.join(IterableSource(self_envs),CobaMultiprocessor(EnvironmentsToObjects(),processes),ObjectsToZipMember(path)).run()
+        CobaContext.logger = CobaContext.logger.undecorate()
+        
         return Environments.from_save(path)
 
     def cache(self) -> 'Environments':
