@@ -1,21 +1,22 @@
 from collections import abc
-from itertools import count, compress, chain, filterfalse, islice
+from itertools import count, compress, chain, filterfalse, islice, repeat
 from typing import Any, Union, Callable, Iterator, Sequence, Mapping, Iterable, Tuple
 from coba.backports import Literal
 
 from coba.primitives import Sparse, Dense, Categorical
+from coba.primitives.rows import Dense_, Sparse_
 from coba.utilities import peek_first
 from coba.pipes.primitives import Filter
 from coba.pipes.filters import Flatten
 
-class LazyDense(Dense):
+class LazyDense(Dense_):
     __slots__ = ('_row','_enc','headers','missing')
 
-    def __init__(self, row: Union[Callable[[],Sequence],Sequence], enc:Sequence = None, hdr:Mapping = None, missing: bool = None) -> None:
+    def __init__(self, row: Union[Callable[[],Sequence],Sequence], enc: Sequence = None, headers: Mapping = None, missing: bool = None) -> None:
         self._row = row
         self._enc = enc
 
-        if hdr is not None: self.headers = hdr
+        if headers is not None: self.headers = headers
         if missing is not None: self.missing = missing
 
     def _load_or_get(self) -> Sequence:
@@ -27,22 +28,43 @@ class LazyDense(Dense):
 
     def __getitem__(self, key: int):
         key = key if key.__class__ is int else self.headers[key]
-        enc = self._enc
         val = self._load_or_get()[key]
+        enc = self._enc
 
-        return val if not enc else enc[key](val)
+        if not enc:
+            return val
+
+        else:
+            try:
+                return enc[key](val)
+            except:
+                if val in ['?','']: return None
+                raise
 
     def __iter__(self) -> Iterator:
-        enc = self._enc
-        if enc:
-            return (e(v) for e,v in zip(enc, self._load_or_get()))
+        if self._enc:
+            return self._enc_all()
         else:
             return iter(self._load_or_get())
 
     def __len__(self) -> int:
         return len(self._load_or_get())
 
-class LazySparse(Sparse):
+    def _enc_all(self)->Iterator:
+        #placing this here gives us a slight perf boost
+        #but is a bit of a conflation of functionality.
+        #In theory this error checking logic with None
+        #should be in an ARFF specific implementation.
+        for e,v in zip(self._enc, self._load_or_get()):
+            try:
+                yield e(v)
+            except:
+                if v in ['?','']: 
+                    yield None
+                else:
+                    raise
+
+class LazySparse(Sparse_):
     __slots__=('_row','_enc','_nsp','_fwd','_inv','missing')
 
     def __init__(self, row: Union[Callable[[],Mapping],Mapping], enc: Mapping = {}, nsp: set = set(), fwd: Mapping = {}, inv: Mapping = {}, missing:bool = None) -> None:
@@ -55,22 +77,29 @@ class LazySparse(Sparse):
 
     def _load_or_get(self) -> Mapping:
         row = self._row
-        if callable(row):
-            row = self._row()
-            self._row = row
-        self._load_or_get = lambda : row
+        if not callable(row): return row
+        row = row()
+        self._row = row
         return row
 
     def __getitem__(self, key: str):
         key = self._fwd.get(key,key)
-        enc = self._enc.get(key,lambda x:x)
-
+        enc = self._enc
+        
         try:
-            return enc(self._load_or_get()[key])
+            val = self._load_or_get()[key]
         except KeyError:
-            if key in self._nsp:
-                return enc("0")
-            raise
+            if key not in self._nsp: raise
+            val = "0"
+
+        if not enc: 
+            return val
+        else:
+            try:
+                return enc.get(key,lambda x:x)(val)
+            except:
+                if val in ['?','']: return None
+                raise
 
     def __iter__(self) -> Iterator:
         return iter(self.keys())
@@ -87,22 +116,29 @@ class LazySparse(Sparse):
         enc = self._enc
         row = self._load_or_get()
 
-        if enc and inv:
-            inv = lambda k: self._inv.get(k,k)
-            t1 = tuple((inv(k), v if k not in enc else enc[k](v)) for k,v in row.items())
-            t2 = tuple((inv(k), enc[k]("0")) for k in self._nsp-row.keys())
-            return t1+t2
-        elif enc and not inv:
-            t1 = tuple((k, v if k not in enc else enc[k](v)) for k,v in row.items())
-            t2 = tuple((k, enc[k]("0")) for k in self._nsp-row.keys())
-            return t1+t2
-        elif not enc and inv:
-            inv = lambda k: self._inv.get(k,k)
-            return tuple((inv(k), v) for k,v in row.items())
+        if enc:
+            return tuple(self._enc_items())
+        elif inv:
+            return tuple((inv.get(k,k), v) for k,v in row.items())
         else:
-            return tuple(row.items())   
+            return tuple(row.items())
+        
+    def _enc_items(self)->Iterator:
+        enc   = self._enc
+        inv   = self._inv
+        raw   = self._load_or_get()
+        items = chain(raw.items(), zip(self._nsp-raw.keys(), repeat("0")))
 
-class HeadDense(Dense):
+        for k,v in items:
+            try:
+                v = enc.get(k,lambda x:x)(v)
+            except:
+                if v in ['?','']: v = None
+                else: raise
+
+            yield (inv.get(k,k) if inv else k,v)
+
+class HeadDense(Dense_):
     __slots__=('_row','headers')
 
     def __init__(self, row: Dense, headers: Mapping[str,int] = None) -> None:
@@ -118,7 +154,7 @@ class HeadDense(Dense):
     def __len__(self) -> int:
         return len(self._row)
 
-class HeadSparse(Sparse):
+class HeadSparse(Sparse_):
 
     __slots__=('_row', '_fwd', '_inv')
 
@@ -163,7 +199,7 @@ class HeadRows(Filter[Iterable[Union[Dense,Sparse]],Iterable[Union[Dense,Sparse]
             mapping_inv = {v:k for k,v in self._mapping.items()}
             yield from (HeadSparse(row, mapping, mapping_inv) for row in rows)
 
-class EncodeDense(Dense):
+class EncodeDense(Dense_):
     __slots__=('_row','_encoders')
 
     def __init__(self, row: Dense, encoders: Sequence) -> None:
@@ -179,7 +215,7 @@ class EncodeDense(Dense):
     def __len__(self) -> int:
         return len(self._encoders)
 
-class EncodeSparse(Sparse):
+class EncodeSparse(Sparse_):
     __slots__=('_row','_enc','_nsp')
 
     def __init__(self, row: Sparse, encoders: Mapping, not_sparse: set) -> None:
@@ -235,7 +271,7 @@ class EncodeRows(Filter[Iterable[Union[Dense,Sparse]],Iterable[Union[Dense,Spars
                 except: pass #pragma: no cover
             return ( EncodeSparse(row, enc, nsp) for row in rows )
 
-class DropOne(Dense):
+class DropOne(Dense_):
     __slots__=('_row','_ind')
 
     def __init__(self, row: Dense, ind: int) -> None:
@@ -254,7 +290,7 @@ class DropOne(Dense):
     def __len__(self) -> int:
         return len(self._row)-1
 
-class KeepDense(Dense):
+class KeepDense(Dense_):
     __slots__=('_row', '_map', '_sel', '_len','headers')
     def __init__(self, row: Dense, mapping: Mapping[Union[str,int],int], selects: Sequence, len: int, headers: Mapping,) -> None:
         self._row = row
@@ -272,7 +308,7 @@ class KeepDense(Dense):
     def __len__(self) -> int:
         return self._len
 
-class DropSparse(Sparse):
+class DropSparse(Sparse_):
     __slots__=('_row','_drop_set')
     
     def __init__(self, row: Sparse, drop_set: set = None) -> None:
@@ -349,15 +385,15 @@ class DropRows(Filter[Iterable[Union[Dense,Sparse]], Iterable[Union[Dense,Sparse
         rows = rows if not self._drop_row else filterfalse(self._drop_row, rows)
 
         if not drop_cols:
-            yield from rows
+            return rows
         elif isinstance(first,Dense):
             mapping, selects, length, headers = DropRows.make_drop_row_args(first, drop_cols)
-            yield from (KeepDense(row, mapping, selects, length, headers) for row in rows)
+            return (KeepDense(row, mapping, selects, length, headers) for row in rows)
         else:
             drop_set = DropRows.make_drop_row_args(first, drop_cols)
-            yield from (DropSparse(row, drop_set) for row in rows)
+            return (DropSparse(row, drop_set) for row in rows)
 
-class LabelDense(Dense):
+class LabelDense(Dense_):
     __slots__=('_row','_ind','_tipe')
 
     def __init__(self, row: Dense, ind: int, tipe: Literal['c','r','m']) -> None:
@@ -390,7 +426,7 @@ class LabelDense(Dense):
     def labeled(self)-> Tuple[Dense,Any,Literal['c','r','m']]:
         return (DropOne(self._row, self._ind), self._row[self._ind], self._tipe)
 
-class LabelSparse(Sparse):
+class LabelSparse(Sparse_):
     __slots__=('_row', '_tipe', '_key')
 
     def __init__(self, row: Sparse, key: str, tipe: Literal['c','r','m']) -> None:
@@ -452,9 +488,9 @@ class LabelRows(Filter[Iterable[Union[Dense,Sparse]],Iterable[Union[Dense,Sparse
 
         if isinstance(first,Dense):
             ind = first.headers[label] if isinstance(label,str) else label
-            return (LabelDense(row, ind, tipe) for row in rows)
+            return map(LabelDense, rows, repeat(ind), repeat(tipe))
         else:
-            return (LabelSparse(row, label, tipe) for row in rows)
+            return map(LabelSparse, rows, repeat(label), repeat(tipe))
 
 class EncodeCatRows(Filter[Iterable[Union[Any,Dense,Sparse]], Iterable[Union[Any,Dense,Sparse]]]):
     def __init__(self, tipe=Literal["onehot","onehot_tuple","string"], value_rows:bool = False) -> None:
@@ -488,7 +524,7 @@ class EncodeCatRows(Filter[Iterable[Union[Any,Dense,Sparse]], Iterable[Union[Any
         elif self._tipe == "string":
             yield from map(str,rows)
         elif "onehot" in self._tipe:
-            for row in rows: yield row.onehot
+            for row in rows: yield row.as_onehot
 
     def _encode_dense_generator(self, rows, first):
         is_mutable = isinstance(first, list)
@@ -502,11 +538,10 @@ class EncodeCatRows(Filter[Iterable[Union[Any,Dense,Sparse]], Iterable[Union[Any
         elif "onehot" in self._tipe:
             for row in rows:
                 row = row.copy() if is_mutable else list(row)
-                for i in cat_cols: row[i] = row[i].onehot
+                for i in cat_cols: row[i] = row[i].as_onehot
                 yield row
 
     def _encode_sparse_generator(self, rows, first):
-        is_mutable = isinstance(first, dict)
         cat_cols = [k for k,v in first.items() if isinstance(v,Categorical) ]
         
         if self._tipe == "string":
@@ -517,5 +552,5 @@ class EncodeCatRows(Filter[Iterable[Union[Any,Dense,Sparse]], Iterable[Union[Any
         elif "onehot" in self._tipe:
             for row in rows:
                 row = row.copy()
-                for k in cat_cols: row[k] = row[k].onehot
+                for k in cat_cols: row[k] = row[k].as_onehot
                 yield row
